import React, { useCallback, useContext, useEffect, useState } from "react";
import { Typography, Link as MuiLink, makeStyles } from "@material-ui/core";
import { SimpleAccordion } from "../common/Accordion/SimpleAccordion";
import { IOCDetails } from "./IOCDetails";
import { Link } from 'react-router-dom';
import { LokiContainer } from "../common/Loki/LokiContainer";
import GitRefLink from "./GitRefLink";
import AlertMessages from "./AlertMessages";
import { IOCService } from "./IOCService";
import AccessControl from '../../components/auth/AccessControl';
import { useDeploymentJobById, userContext } from "../../api/SwaggerApi";
import { DeploymentStatus, IocActiveDeployment } from "../../api/DataTypes";
import { theme } from "../../style/Theme";
import { JobAsyncList } from "../Job/JobAsyncList";

const useStyles = makeStyles((theme) => ({
  iocNotDeployed: {
    color: theme.iocNotDeployed.color
  }
}));

export function IOCLiveStatus({ ioc, currentCommand, commands, getCommands, commandsLoading, buttonDisabled, setButtonDisabled,
  commandLazyParams, setCommandLazyParams, commandColumnSort, setCommandColumnSort, rowsPerPage }) {
  const { user } = useContext(userContext);
  const [deploymentJob, getDeploymentJobById] = useDeploymentJobById();
  const [asyncCommands, setAsyncCommands] = useState(commands.operationsList);

  const classes = useStyles(theme);

  useEffect(() => {
    if (ioc.activeDeployment?.awxJobId) {
      getDeploymentJobById(ioc.activeDeployment?.awxJobId);
    }
  }, [getDeploymentJobById, ioc.activeDeployment?.awxJobId])

  useEffect(() => {
    setAsyncCommands(commands.operationsList);
  }, [commands.operationsList])

  const liveIOC = { ...ioc };
  liveIOC.name = ioc.namingName;

  const getSubset = useCallback((ioc) => {

    const iocHascDeployment = new IocActiveDeployment(ioc.activeDeployment).hasDeployment();

    const grayOutNoText = { shownValue: '---', shownClass: classes.iocNotDeployed };

    //if IOC is not deployed some fields has to show '---' value
    if (!iocHascDeployment) {
      var notDeployedText = grayOutNoText;
    }


    //Show START/STOP components only when IOC was deployed SUCCESSFULLY
    const deploymentStatus = new DeploymentStatus(ioc.activeDeployment, deploymentJob);
    const showControls = deploymentStatus.wasSuccessful();
    let subset = {
      "IOC name":
        <Typography><MuiLink href={`${window.CCDB_ADDRESS}?name=${ioc.namingName}`}
          target="_blank" rel="noreferrer">{ioc.namingName}</MuiLink></Typography>,
      "Description": ioc.description,
      "Owner": ioc.owner,
      "Git reference": iocHascDeployment ? <GitRefLink ioc={ioc} /> : notDeployedText,
      "Deployed on": iocHascDeployment ?
        <Typography>
          {ioc.activeDeployment?.host.csEntryId ?
            <Link to={`/hosts/${ioc.activeDeployment?.host.csEntryId}`}>{ioc.activeDeployment?.host.fqdn}</Link>
            :
            <>{ioc.activeDeployment?.host.fqdn}</>
          }
        </Typography>
        : notDeployedText,
        "Deployment comment": grayOutNoText
    }

    //show comment only for deployments and where comment field is filled
    if (ioc.activeDeployment && !ioc.activeDeployment.isUndeployment) {
      subset['Deployment comment'] = ioc.activeDeployment?.comment?.length > 0 ? ioc.activeDeployment?.comment : grayOutNoText;
    }

    if (user) {
      subset["IOC Service Controls"] = showControls ?
        <IOCService {...{ ioc, currentCommand, getCommands, buttonDisabled, setButtonDisabled, commandColumnSort, commandLazyParams }} />
        : "Can't be controlled, because IOC is not (yet) deployed succesfully";
    }

    return subset
  }, [buttonDisabled, commandColumnSort, commandLazyParams, currentCommand, deploymentJob, getCommands, setButtonDisabled, user, classes.iocNotDeployed]);

  function isIocDeployed(ioc) {
    return Boolean(ioc.activeDeployment);
  }

  return (
    <>
      <IOCDetails ioc={liveIOC} getSubset={getSubset} alert={<AlertMessages alerts={liveIOC.alerts} />} />
      <SimpleAccordion summary="IOC Service Control log">
        <JobAsyncList
          jobs={asyncCommands} rowType="iocServiceControl" setJobs={setAsyncCommands} loading={commandsLoading}
          totalCount={commands.totalCount} lazyParams={commandLazyParams} setLazyParams={setCommandLazyParams} columnSort={commandColumnSort} setColumnSort={setCommandColumnSort}
          rowsPerPage={rowsPerPage} />
      </SimpleAccordion>
      {(liveIOC.activeDeployment?.host?.csEntryIdValid) &&
        <AccessControl allowedRoles={["DeploymentToolAdmin", "DeploymentToolIntegrator"]}
          renderNoAccess={() => <></>}>
          <SimpleAccordion summary="ProcServLog info" defaultExpanded>
            <LokiContainer csEntryId={liveIOC.activeDeployment?.host.csEntryId} iocName={ioc.namingName} isDeployed={isIocDeployed(ioc)} />
          </SimpleAccordion>
        </AccessControl>
      }
    </>
  );
}
