import React, { useCallback } from 'react';
import { CustomTable } from "../common/table/CustomTable";
import { Grid, Tooltip } from "@material-ui/core";
import { IOCStatusIcon } from './IOCIcons';
import { noWrapText } from '../common/Helper';
import { useRedirect } from '../../hooks/Redirect';

const ownIocsColumns = [
  { id: "status", label: 'Status', width: '8ch', textAlign: "center" },
  { id: "namingName", label: 'IOC Name', width: '15ch', sortable: true },
  { id: "host", label: 'Host', width: '10ch', sortable: false },
  { id: "network", label: 'Network', width: '10ch', sortable: false },
  { id: "sourceVersion", label: 'Reference', width: '12ch', sortable: false },
]

const exploreIocsColumns = [
  { id: "status", label: 'Status', width: '8ch', textAlign: "center" },
  { id: "namingName", label: 'IOC Name', width: '15ch', sortable: true },
  { id: "host", label: 'Host', width: '10ch', sortable: false },
  { id: "network", label: 'Network', width: '10ch', sortable: false },
  { id: "owner", label: 'Owner', width: '13ch', sortable: true },
  { id: "sourceVersion", label: 'Reference', width: '12ch', sortable: false },
]

const hostDetailsColumns = [
  { id: "status", label: 'Status', width: '8ch', textAlign: "center" },
  { id: "namingName", label: 'IOC Name', width: '15ch', sortable: true },
  { id: "sourceVersion", label: 'Reference', width: '12ch', sortable: false },
]

function createGitVersionField(version, shortVersion) {
  return (
    <>
      {version ?
        <Tooltip title={version} arrow enterDelay={400}>
          <label>{noWrapText(shortVersion ?? version)}</label>
        </Tooltip>
        : "---"
      }
    </>
  )
}



function createTableRowForHostDetails(ioc) {
  const { id, name, namingName, sourceVersion, sourceVersionShort, alert } = ioc;

  return {
    id: id,
    status:
      <Grid container direction="column" justifyContent="center" alignItems="center">
        <IOCStatusIcon ioc={ioc} />
      </Grid>,
    namingName: noWrapText(namingName ?? name),
    sourceVersion: createGitVersionField(sourceVersion, sourceVersionShort),
    discrepancy: alert === "WARNING",
    inconsistentState: alert === "ERROR",
  };
}

function createTableRowForOwnIocs(ioc) {
  const { id, activeDeployment, namingName, sourceVersion, sourceVersionShort, alert } = ioc;

  return {
    id: id,
    status:
      <Grid container direction="column" justifyContent="flex-start" alignItems="center">
        <IOCStatusIcon ioc={ioc} />
      </Grid>,
    namingName: noWrapText(namingName ?? "---"),
    sourceVersion: createGitVersionField(sourceVersion, sourceVersionShort),
    host: noWrapText(activeDeployment?.host.hostName || "---"),
    network: noWrapText(activeDeployment?.host.network || "---"),
    discrepancy: alert === "WARNING",
    inconsistentState: alert === "ERROR",
    iocNotDeployed: !ioc.activeDeployment
  };
}

function createTableRowForExploreIocs(ioc) {
  const { id, namingName, owner, sourceVersion, sourceVersionShort, alert, host } = ioc;
  return {
    id: id,
    status:
      <Grid container direction="column" justifyContent="center" alignItems="center">
        <IOCStatusIcon ioc={ioc} />
      </Grid>,
    namingName: noWrapText(namingName ?? "---"),
    sourceVersion: createGitVersionField(sourceVersion, sourceVersionShort),
    host: noWrapText(host?.hostName || "---"),
    network: noWrapText(host?.network || "---"),
    owner: noWrapText(owner || "---"),
    discrepancy: alert === "WARNING",
    inconsistentState: alert === "ERROR",
    iocNotDeployed: !ioc.activeDeployment
  };
}

export function IOCTable({ iocs, rowType = "own", totalCount, lazyParams, setLazyParams, columnSort, setColumnSort, rowsPerPage, loading, paginator }) {
  const tableTypeSpecifics = {
    "own": [ownIocsColumns, createTableRowForOwnIocs],
    "host": [hostDetailsColumns, createTableRowForHostDetails],
    "explore": [exploreIocsColumns, createTableRowForExploreIocs]
  }

  const redirect = useRedirect();
  const [columns, createRow] = tableTypeSpecifics[rowType];
  const rows = iocs.map(ioc => createRow(ioc));

  const itemLink = id => `/iocs/${id}`;

  const onRowClicked = useCallback((id) => {
    redirect(itemLink(id));
  }, [redirect]);

  return (
    <CustomTable
      columns={columns}
      totalCount={totalCount}
      rows={rows}
      handleRowClick={onRowClicked}
      itemLink={itemLink}
      lazyParams={lazyParams}
      setLazyParams={setLazyParams}
      columnSort={columnSort}
      setColumnSort={setColumnSort}
      rowsPerPage={rowsPerPage}
      loading={loading}
      paginator={paginator}
    />
  );
}
