import React, { useEffect, useState } from 'react';
import { Grid, Typography, Card, CardContent, Container, Link as MuiLink } from '@material-ui/core';
import { DeploymentStepper } from "./DeploymentStepper"
import { KeyValueTable } from "../common/KeyValueTable/KeyValueTable";
import { SimpleAccordion } from "../common/Accordion/SimpleAccordion";
import { DeploymentBadge } from './DeploymentBadge';
import { DeploymentJobOutput } from './DeploymentJobOutput';
import { Alert } from '@material-ui/lab';
import { Link } from 'react-router-dom';
import { formatDate } from '../common/Helper';
import GitRefLink from '../IOC/GitRefLink';
import AccessControl from '../auth/AccessControl';
import { DeploymentStatus } from '../../api/DataTypes';

function createAlert(deployment, deploymentJob) {
  const deploymentStatus = new DeploymentStatus(deployment, deploymentJob);
  const message = deploymentStatus.message();
  const severity = deploymentStatus.severity();
  return <Alert severity={severity} variant="standard">{message}</Alert>;
}

export function DeploymentDetails({ deployment, deploymentJob }) {
  const [alert, setAlert] = useState(createAlert(deployment, deploymentJob));

  useEffect(() => {
    setAlert(createAlert(deployment, deploymentJob));

  }, [deployment, deploymentJob])

  const deploymentDetails = {
    "Git reference": <GitRefLink ioc={deployment} />,
    host:
      <Typography>
        {deployment.host.csEntryId ?
          <Link to={`/hosts/${deployment.host.csEntryId}`}>{deployment.host.fqdn}</Link>
          :
          <>{deployment.host.fqdn}</>
        }
      </Typography>,
    comment: deployment.comment?.length > 0 ? deployment.comment : "---",
    repository: <Typography><MuiLink href={deployment.sourceUrl} target="_blank" rel="noreferrer">{deployment.sourceUrl}</MuiLink></Typography>,
    "AWX job link": <Typography><MuiLink href={deployment.awxJobUrl} target="_blank" rel="noreferrer">{deployment.awxJobUrl}</MuiLink></Typography>,
    "start time": deploymentJob?.started ? formatDate(deploymentJob.started) : "-",
    "end time": deploymentJob?.finished ? formatDate(deploymentJob.finished) : "-"
  }

  return (
    <Grid container spacing={1}>
      <Grid item xs={12} md={12}>
        <SimpleAccordion defaultExpanded summary={<DeploymentBadge deployment={deployment} />}>
          <KeyValueTable obj={deploymentDetails} variant="overline" />
        </SimpleAccordion>
      </Grid>
      <Grid item xs={12} md={12}>
        <Card >
          <CardContent>
            <div>
              {deploymentJob &&
                <DeploymentStepper
                  activeStep={["created", "pending", "running", "successful"].indexOf(deploymentJob.status.toLowerCase())}
                  deploymentStatus={deploymentJob.status}
                  deploymentStart={deployment.startDate}
                  undeployment={deployment.undeployment}
                />}
            </div>
          </CardContent>
        </Card>
      </Grid>
      {alert && <Grid item xs={12}>
        {alert}
      </Grid>}
      <AccessControl allowedRoles={["DeploymentToolAdmin", "DeploymentToolIntegrator"]}
        renderNoAccess={() => <></>}>
        <Grid item xs={12} md={12} style={{ paddingBottom: 0 }}>
          {deploymentJob ?
            <SimpleAccordion summary="Ansible Job Output" defaultExpanded>
              <Container>
                <DeploymentJobOutput deploymentJob={deploymentJob} />
              </Container>
            </SimpleAccordion>
            :
            <></>
          }
        </Grid>
      </AccessControl>
    </Grid>
  );
}

